/*
 * Jednoduche kresleni v Jave.
 * 
 * Jiri Skala, brezen 2008
 * 
 */
package simpleDrawing;

import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

import javax.swing.event.MouseInputAdapter;

/**
 * Panel do ktereho budeme kreslit.
 * 
 * @author JSkala
 */
class PanelCanvas extends javax.swing.JPanel {
	
	/**
	 * Velikost krizku.
	 */
	private final int crossSize = 5;
	
	/**
	 * Tolerance pri chytani krizku mysi.
	 */
	private final int mouseTolerance = 7;
	
	
	
	
	
	/**
	 * Kliknutim se krizek smaze?
	 * Pokud ne, bude se posouvat.
	 */
	private boolean clickDeletes = false;
	
	/**
	 * Seznam krizku.
	 */
	private ArrayList<Point> crosses;
	
	/**
	 * Index krizku, ktery tahneme.
	 * Hodnota -1 znamena, ze zadny netahneme.
	 */
	private int draggedCrossIndex = -1;
	
	
	
	
	
	/**
	 * Konstruktor.
	 */
	public PanelCanvas() {
		
		// zavolame konstruktor panelu
		super();
		
		// zaregistrujeme obsluhu udalosti
		PanelMouseInputAdapter pmia = new PanelMouseInputAdapter();
		addMouseListener(pmia);
		addMouseMotionListener(pmia);
		
		// pripravime seznam
		crosses = new ArrayList<Point>();
	}
	
	/**
	 * Nastavi zda se ma krizek pri kliknuti smazat.
	 * 
	 * @param del True pokud se ma krizek smazat, jinak false.
	 */
	public void setClickDeletes(boolean clickDeletes) {
		
		this.clickDeletes = clickDeletes;
	}
	
	/**
	 * Metoda ktera se vyvola pri kresleni panelu.
	 * Nevolat primo, pouzivat repaint().
	 * 
	 * @param g Objekt pro kresleni.
	 */
	public void paint(Graphics g) {
	
		// pretypujeme Graphics na namaklejsi Graphics2D
		Graphics2D gr = (Graphics2D)g;
		
		// zjistime velikost panelu
		int w = this.getWidth();
		int h = this.getHeight();
		
		// nastavime barvu pro kresleni (obrys i vypln)
		gr.setColor(Color.black);
		// nastavime barvu pozadi
		gr.setBackground(Color.white);
		
		// smazeme obsah panelu
		gr.clearRect(0, 0, w-1, h-1);
		
		// nastavime tlustejsi caru
		gr.setStroke(new BasicStroke(3));
		
		// nakreslime krizky
		for (int i = 0; i < crosses.size(); i++)
			drawCross(gr, crosses.get(i));
	}
	
	/**
	 * Nakresli krizek.
	 * 
	 * @param gr	Graphics pro kresleni.
	 * @param p		Misto kam se ma krizek nakreslit.
	 */
	private void drawCross(Graphics2D gr, Point p) {
		
		int left = p.x - crossSize;
		int right = p.x + crossSize;
		int top = p.y - crossSize;
		int bottom = p.y + crossSize;
		
		gr.drawLine(left, top, right, bottom);
		gr.drawLine(left, bottom, right, top);
	}
	
	/**
	 * Zkusi najit krizek, ktery lezi na zadanem miste.
	 * 
	 * @param querry	Misto kde hledat.
	 * @return			Vraci index krizku, nebo -1 pokud na zadanem miste zadny krizek neni.
	 */
	private int findCross(Point querry) {
		
		// prohledame krizky
		for (int i = 0; i < crosses.size(); i++)
			// zjistime zda je nektery pobliz
			if (pointsClose(querry, crosses.get(i)))
				// nasli jsme!
				return i;
		
		// pokud jsme se dostali az sem, zadny krizek jsme nenasli
		return -1;
	}
	
	/**
	 * Zjisti zda jsou zadane dva body blizko u sebe.
	 * 
	 * @param pa	Prvni bod.
	 * @param pb	Druhy bod.
	 * @return		Vraci true pokud jsou zadane body blizko u sebe, jinak vraci false.
	 */
	private boolean pointsClose(Point pa, Point pb) {
		
		int dx = pa.x - pb.x;
		int dy = pa.y - pb.y;
		
		// nebudeme odmocnovat, zbytecne zdrzeni
		int dSqr = dx*dx + dy*dy;
		
		// misto odmocnovani porovname s druhou mocninou tolerance
		return dSqr <= mouseTolerance*mouseTolerance;
	}
	
	
	
	
	
	/**
	 * Trida pro obsluhu udalosti mysi na kreslicim platne.
	 * 
	 * @author Jiri Skala
	 */
	private class PanelMouseInputAdapter extends MouseInputAdapter {
		
		/**
		 * Reakce na kliknuti.
		 * 
		 * @param e Informace o udalosti.
		 */
		public void mouseClicked(MouseEvent e) {
			
			// kliknuto levym tlacitkem?
			if (e.getButton() == MouseEvent.BUTTON1) {
				// pridame krizek na soucasnou pozici mysi
				crosses.add(e.getPoint());
				// prekreslime
				repaint();
			}
			// kliknuto pravym tlacitkem?
			else if (e.getButton() == MouseEvent.BUTTON3) {
				int index;
				// mame smazat? je pobliz nejaky krizek?
				if (clickDeletes && (index = findCross(e.getPoint())) >= 0) {
					// smazeme krizek
					crosses.remove(index);
					// prekreslime
					repaint();
				}
			}
		}
		
		/**
		 * Reakce na stisknuti tlacitka mysi.
		 */
		public void mousePressed(MouseEvent e) {
			
			// chceme tahat?
			if (e.getButton() == MouseEvent.BUTTON3 && ! clickDeletes)
				// ano, zkusime najit krizek
				draggedCrossIndex = findCross(e.getPoint());
		}
		
		/**
		 * Reakce na uvolneni tlacitka mysi.
		 */
		public void mouseReleased(MouseEvent e) {
			
			// pokud se uvolnilo prave tlacitko
			if (e.getButton() == MouseEvent.BUTTON3)
				// prestaneme tahat
				draggedCrossIndex = -1;
		}
		
		/**
		 * Reakce na pohyb mysi SE STISKNUTYM TLACITKEM.
		 */
		public void mouseDragged(MouseEvent e) {
			
			// pokud tahneme nejaky krizek
			if (draggedCrossIndex >= 0) {
				// aktualizujeme jeho polohu
				crosses.set(draggedCrossIndex, e.getPoint());
				// prekreslime
				repaint();
			}
		}
	}
}
